/*
Subqueries - this is our starting point today as it ties to most of the 
    topics that are covered.  
A subquery is just a query inside of another query.  
Mostly used to add a new column or create a filter.  
Subqueries are always between parentheses and can be in different places within the main query, 
	usually in the SELECT, FROM or WHERE clauses.  
	Technically speaking you can use unlimited subqueries and you can have many nested queries.

That said, Subqueries can impact performace, so that is something to be aware of.
*/

--Here is a simple example of using a subquery to return matching rows.
select tst.* FROM tst where pid in (select id from stu where del = 0 and sc = 994)

-- using a subquery as a nested table 
-- let's SELECT the most recent ENR record
select enr.* 
from enr inner join 
	(select subENR.id, 
			max(subENR.ed) as maxDT 
	 from enr as subENR where del = 0 group by subENR.id) as e 
	 on enr.id = e.id and enr.ed = e.maxDT 
where del = 0

/* ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ*/

--Using subqueries as fields which is probably one of the more common use cases here we 
--are pulling three subqueries.  You can pull as many subqueries as you want, but the 
--more you pull the greater the impact on the Server.  
select stu.id, stu.ln, lf, 
(select max(dt) from ads where ads.del = 0 and ads.pid = stu.id) as RecentADSdate,
(select min(dt) from ads where ads.del = 0 and ads.pid = stu.id) as FirstADSDate,
(select de from cod where tc = 'stu' and fc = 'lf' and cod.cd = stu.lf) as lf
from stu
where del = 0

--The same subquery could still be used but using a join on ads to make our query slightly more efficient:
select stu.id, stu.ln, stu.lf, max(ads.dt) as RecentADSdate, min(ads.dt) as FirstADSDate,
(select de from cod where tc = 'stu' and fc = 'lf' and cod.cd = stu.lf) as lf
from stu left join ads on stu.id = ads.pid 
where stu.del = 0 
group by stu.id, stu.ln, stu.lf
order by stu.id

/* ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ*/
--list account ids that have more than 3 students associated with it
select pwa.aid, count(*) 
from pwa inner join pws on pwa.aid = pws.aid
where pwa.del = 0 and pws.del = 0 and pwa.ty ='P'
group by pwa.aid 
having count(*) >= 2

-- display PWA records that have more than 3 students associated with it.
-- in this example we are using the subquery in the WHERE clause with 
-- the keyword IN to allow us to pick up only those records that we choose
select * 
from pwa 
where del = 0 and ty = 'P' and aid in (select aid from pws where del = 0 group by aid having count(*) > 3)


/*  As mentioned, Subqueries can be used in a WHERE clause and with keywords IN or EXISTS.  
Subqueries can be used to pick up a single piece of data from one table in order to update a value in another table.  
So those are some of the reasons you may use a subquery.  
Be mindful each time that data is being pulled the subquery is being run, it could add up and using a join may be a better way. */

