
/*  When we look at joining data sets we sometimes want to see two sets of data together.  
	For example, maybe we want to pull a listing of staff who are at the district office, 
	and a group of staff who are parents.  
	We are going to simplify it a bit for the sake of time for the demonstration 
	to show the use of UNION, UNION ALL, INTERSECT and EXCEPT
Here are the rules to UNION data:
	Each query must have the same number of columns
	Each column must have compatible data types
	Column names for the final result set are taken from the first query
	ORDER BY and COMPUTE clauses can only be issued for the overall result set and not within each individual result set
	GROUP BY and HAVING clauses can only be issued for each individual result set and not for the overall result set */

SELECT FN, LN FROM STF
UNION
SELECT FN, LN FROM CON WHERE RL in ('10','11') --12192

SELECT FN, LN FROM STF
UNION ALL
SELECT FN, LN FROM CON WHERE RL in ('10','11') --12323

/* Note that the UNION ALL has more records as it removes any duplicate records */

--Now what if we want to see who is in both tables?  Using a standard TSQL command with an inner join it would be: */
SELECT CON.FN, CON.LN 
FROM CON INNER JOIN STF ON CON.FN = STF.FN AND CON.LN = STF.LN
WHERE CON.RL in ('10','11')
GROUP BY CON.FN, CON.LN

/* Now using INTERSECT: Note that the queries are more aligned with the UNION and UNION ALL and make for an easier transition
   over using the same query with a join.  
   This is just a simple example and could have been more detailed, but it lets you see the concept.
   */

SELECT FN, LN FROM CON where RL in ('10','11')  
INTERSECT
SELECT FN, LN FROM STF 


EXCEPT: 
SELECT FN, LN FROM CON where RL in ('10','11')  
EXCEPT 
SELECT FN, LN FROM STF 

/* To write a query that would give us the same results for the EXCEPT we would 
   need to do something like below, which takes more coding and this is a simple example, 
   imagine if you were also linking up phone number, address and other fields: */
   
SELECT C.FN, C.LN 
FROM CON C
WHERE NOT EXISTS (SELECT FN, LN FROM STF s
			 WHERE C.FN = S.FN 
			 AND C.LN = S.LN)
GROUP BY C.FN, C.LN				 

/* ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ*/

/* CROSS APPLY: The CROSS APPLY operator returns rows from the left table expression (in its final output) 
	if it matches the right table expression. 
	In other words, when matched, the right table expression returns rows for the left expression. 
	It's helpful to think of a CROSS APPLY as equivalent to an INNER JOIN 
	(or, more precisely, it's like a CROSS JOIN with a correlated sub-query) 
	with an implicit join condition of 1=1.

	Use a CROSS APPLY when no easy join exists and when the right table is an expression or table-valued function. 
	It's like an INNER JOIN since rows must exist in both tables/expressions for SQL to return results. 
	
	The most common use for this that I have used and seen if in pulling the Get_Description_New function which
	is a part of each Aeries database.  This allows us to pull the description of a field instead of using a 
	subquery 
	
	*/

SELECT 
LOC.NM  [Site Name],
STU.FN [ChildFirstName],
STU.MN [ChildMiddleName],
STU.LN [ChildLastName],
CONVERT(VARCHAR,STU.BD,101) [Birthday],
CONVERT(VARCHAR,STU.ED,101) [Enrolled Date],
[GD_STU_RC1_STU_RC1].[DE] [ChildRace],
	CASE
		WHEN CSE.ID IS NULL THEN 'No' ELSE 'Yes'
	END [iep/ifsp]
FROM STU 
	INNER JOIN (SELECT * FROM LOC WHERE DEL = 0) LOC ON STU.SC = LOC.CD
	 LEFT JOIN (SELECT * FROM CSE 
					WHERE CSE.DEL = 0 
					AND CSE.DI <> '' 
					AND CSE.ED <= GETDATE() 
					AND (CSE.XD IS NULL OR CSE.XD >= GETDATE())) CSE ON STU.ID = CSE.ID
CROSS APPLY dbo.Get_Description_New(98, 'STU', 'RC1', [STU].[RC1]) AS [GD_STU_RC1_STU_RC1]
WHERE STU.DEL = 0 AND STU.TG = '' 

/* Once more I wanted to stress this was just an introduction to the use of INTERSECT, EXCEPT, UNION, UNION ALL,
   and the use of CROSS APPLY.  There are other JOINs that can be done to pull the data together and
   depending on the circumstances these or a subquery will help round out your skill set 
   
*/
